@@ -62,8 +62,6 @@ public:
 
     void operator()(mir::Server& server);
 
-    ///\deprecated use the better name: process_outputs()
-    void for_each_output(std::function<void(std::vector<Output> const& outputs)> const& functor) const __attribute__ ((deprecated));
     void process_outputs(std::function<void(std::vector<Output> const& outputs)> const& functor) const;
 
 private:
