@@ -19,9 +19,12 @@
 #ifndef MIRAL_WINDOW_MANAGEMENT_POLICY_H
 #define MIRAL_WINDOW_MANAGEMENT_POLICY_H
 
+#include <mir/geometry/displacement.h>
 #include <mir/geometry/rectangles.h>
 #include <mir_toolkit/event.h>
 
+#include <mir/client/detail/mir_forward_compatibility.h>
+
 namespace miral
 {
 class Window;
@@ -43,11 +46,11 @@ public:
 
     /** Customize initial window placement
      *
-     * @param app_info                the application requesting a new surface
+     * @param app_info                the application requesting a new window
      * @param requested_specification the requested specification (updated with default placement)
      * @return                        the customized specification
      */
-    virtual auto place_new_surface(
+    virtual auto place_new_window(
         ApplicationInfo const& app_info,
         WindowSpecification const& requested_specification) -> WindowSpecification = 0;
 
@@ -127,20 +130,20 @@ public:
      *
      * @param window_info   the window
      */
-    virtual void advise_focus_lost(WindowInfo const& info);
+    virtual void advise_focus_lost(WindowInfo const& window_info);
 
     /** Notification that a window has gained focus
      *
      * @param window_info   the window
      */
-    virtual void advise_focus_gained(WindowInfo const& info);
+    virtual void advise_focus_gained(WindowInfo const& window_info);
 
     /** Notification that a window is about to change state
      *
      * @param window_info   the window
      * @param state         the new state
      */
-    virtual void advise_state_change(WindowInfo const& window_info, MirSurfaceState state);
+    virtual void advise_state_change(WindowInfo const& window_info, MirWindowState state);
 
     /** Notification that a window is about to move
      *
@@ -172,6 +175,15 @@ public:
     virtual void advise_raise(std::vector<Window> const& windows);
 /** @} */
 
+    /** Confirm (and optionally adjust) the motion of a child window when the parent is moved.
+     *
+     * @param window_info   the window
+     * @param movement      the movement of the parent
+     *
+     * @return the confirmed placement of the window
+     */
+    virtual auto confirm_inherited_move(WindowInfo const& window_info, Displacement movement) -> Rectangle = 0;
+
     virtual ~WindowManagementPolicy() = default;
     WindowManagementPolicy() = default;
     WindowManagementPolicy(WindowManagementPolicy const&) = delete;
