@@ -25,7 +25,10 @@
 #include <mir/server.h>
 #include <mir/version.h>
 
-#if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 24, 1)
+#if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 26, 0)
+#include <mir/input/keymap.h>
+#include <mir/input/mir_keyboard_config.h>
+#elif MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 24, 1)
 #include <mir/input/keymap.h>
 #include <mir/input/keyboard_configuration.h>
 #endif
@@ -112,7 +115,11 @@ struct miral::Keymap::Self : mir::input::InputDeviceObserver
 
         if (keyboard_config.is_set())
         {
+#if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 26, 0)
+            keymap = keyboard_config.value().device_keymap();
+#else
             keymap = keyboard_config.value().device_keymap;
+#endif
         }
 
         keymap.layout = layout;
