@@ -1,5 +1,5 @@
 /*
- * Copyright © 2016 Canonical Ltd.
+ * Copyright © 2016-2017 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -37,17 +37,21 @@ int main(int argc, char const* argv[])
 {
     using namespace miral;
 
+    std::function<void()> shutdown_hook{[]{}};
+
     SpinnerSplash spinner;
     InternalClientLauncher launcher;
     ActiveOutputsMonitor outputs_monitor;
     WindowManagerOptions window_managers
         {
-            add_window_manager_policy<TitlebarWindowManagerPolicy>("titlebar", spinner, launcher),
+            add_window_manager_policy<TitlebarWindowManagerPolicy>("titlebar", spinner, launcher, shutdown_hook),
             add_window_manager_policy<TilingWindowManagerPolicy>("tiling", spinner, launcher, outputs_monitor),
         };
 
     MirRunner runner{argc, argv};
 
+    runner.add_stop_callback([&] { shutdown_hook(); });
+
     auto const quit_on_ctrl_alt_bksp = [&](MirEvent const* event)
         {
             if (mir_event_get_type(event) != mir_event_type_input)
@@ -78,7 +82,7 @@ int main(int argc, char const* argv[])
     return runner.run_with(
         {
             CommandLineOption{[&](std::string const& ) { },
-                              "desktop_file_hint", "Ignored for Unity8 compatability", "miral-shell.desktop"},
+                              "desktop_file_hint", "Ignored for Unity8 compatibility", "miral-shell.desktop"},
             CursorTheme{"default"},
             window_managers,
             display_configuration_options,
