@@ -15,17 +15,21 @@
  */
 
 #include "miregl.h"
+#include <mir/client/window_spec.h>
+#include <mir_toolkit/version.h>
 
 #include <cstring>
 
 #include <GLES2/gl2.h>
+#include <miral/window_specification.h>
+#include <mir/client/window_spec.h>
 
 class MirEglApp
 {
 public:
     MirEglApp(MirConnection* const connection, MirPixelFormat pixel_format);
 
-    EGLSurface create_surface(MirSurface* surface);
+    EGLSurface create_surface(MirWindow* window);
 
     void make_current(EGLSurface eglsurface) const;
 
@@ -58,37 +62,35 @@ std::shared_ptr<MirEglApp> make_mir_eglapp(
 
 namespace
 {
-MirSurface* create_surface(MirConnection* const connection, MirSurfaceParameters const& surfaceparm)
+MirWindow* create_window(MirConnection* const connection, MirWindowParameters const& parameters)
 {
-    auto const spec = mir_connection_create_spec_for_normal_surface(
-        connection,
-        surfaceparm.width,
-        surfaceparm.height,
-        surfaceparm.pixel_format);
 
-    mir_surface_spec_set_name(spec, surfaceparm.name);
-    mir_surface_spec_set_buffer_usage(spec, surfaceparm.buffer_usage);
+    auto spec = mir::client::WindowSpec::for_normal_window(
+        connection, parameters.width, parameters.height, parameters.pixel_format)
+        .set_name(parameters.name)
+        .set_buffer_usage(parameters.buffer_usage);
 
-    if (!surfaceparm.width && !surfaceparm.height)
-        mir_surface_spec_set_fullscreen_on_output(spec, surfaceparm.output_id);
 
-    auto const surface = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    if (!parameters.width && !parameters.height)
+        spec.set_fullscreen_on_output(parameters.output_id);
 
-    if (!mir_surface_is_valid(surface))
-        throw std::runtime_error(std::string("Can't create a surface ") + mir_surface_get_error_message(surface));
+    auto const window = mir_create_window_sync(spec);
 
-    if (surfaceparm.output_id != mir_display_output_id_invalid)
-        mir_surface_set_state(surface, mir_surface_state_fullscreen);
+    if (!mir_window_is_valid(window))
+        throw std::runtime_error(std::string("Can't create a window ") + mir_window_get_error_message(window));
 
-    return surface;
+    if (parameters.output_id != mir_display_output_id_invalid)
+        mir_window_set_state(window, mir_window_state_fullscreen);
+
+    return window;
 }
 }
 
-MirEglSurface::MirEglSurface(std::shared_ptr<MirEglApp> const& mir_egl_app, MirSurfaceParameters const& surfaceparm, int swapinterval) :
+MirEglSurface::MirEglSurface(
+    std::shared_ptr<MirEglApp> const& mir_egl_app, MirWindowParameters const& parm, int swapinterval) :
     mir_egl_app{mir_egl_app},
-    surface{create_surface(mir_egl_app->connection, surfaceparm)},
-    eglsurface{mir_egl_app->create_surface(surface)},
+    window{create_window(mir_egl_app->connection, parm)},
+    eglsurface{mir_egl_app->create_surface(window)},
     width_{0},
     height_{0}
 {
@@ -98,7 +100,7 @@ MirEglSurface::MirEglSurface(std::shared_ptr<MirEglApp> const& mir_egl_app, MirS
 MirEglSurface::~MirEglSurface()
 {
     mir_egl_app->destroy_surface(eglsurface);
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
 }
 
 void MirEglSurface::egl_make_current()
@@ -173,7 +175,7 @@ MirEglApp::MirEglApp(MirConnection* const connection, MirPixelFormat pixel_forma
              EGL_HEIGHT, 1,
              EGL_NONE
         };
-     
+
         dummy_surface = eglCreatePbufferSurface(egldisplay, eglconfig, dummy_pbuffer_attribs);
         if (dummy_surface == EGL_NO_SURFACE)
             throw std::runtime_error("eglCreatePbufferSurface failed");
@@ -182,12 +184,12 @@ MirEglApp::MirEglApp(MirConnection* const connection, MirPixelFormat pixel_forma
     make_current(dummy_surface);
 }
 
-EGLSurface MirEglApp::create_surface(MirSurface* surface)
+EGLSurface MirEglApp::create_surface(MirWindow* window)
 {
     auto const eglsurface = eglCreateWindowSurface(
         egldisplay,
         eglconfig,
-        (EGLNativeWindowType) mir_buffer_stream_get_egl_native_window(mir_surface_get_buffer_stream(surface)), NULL);
+        (EGLNativeWindowType) mir_buffer_stream_get_egl_native_window(mir_window_get_buffer_stream(window)), NULL);
 
     if (eglsurface == EGL_NO_SURFACE)
         throw std::runtime_error("eglCreateWindowSurface failed");
