@@ -17,9 +17,9 @@
  */
 
 #include "test_server.h"
+#include "../miral/basic_window_manager.h"
 
-#include <miral/canonical_window_manager.h>
-#include <miral/set_window_managment_policy.h>
+#include <miral/set_window_management_policy.h>
 
 #include <mir_test_framework/executable_path.h>
 #include <mir_test_framework/stub_server_platform_factory.h>
@@ -39,6 +39,7 @@
 using namespace miral;
 using namespace testing;
 namespace mtf = mir_test_framework;
+namespace msh = mir::shell;
 
 namespace
 {
@@ -46,6 +47,14 @@ std::chrono::seconds const timeout{20};
 char const* dummy_args[2] = { "TestServer", nullptr };
 }
 
+miral::TestServer::TestWindowManagerPolicy::TestWindowManagerPolicy(
+    WindowManagerTools const& tools, TestServer& test_fixture) :
+    CanonicalWindowManagerPolicy{tools}
+{
+    test_fixture.tools = tools;
+    test_fixture.policy = this;
+}
+
 miral::TestServer::TestServer() :
     runner{1, dummy_args}
 {
@@ -54,6 +63,12 @@ miral::TestServer::TestServer() :
     add_to_environment("MIR_SERVER_NO_FILE", "on");
 }
 
+auto miral::TestServer::build_window_manager_policy(WindowManagerTools const& tools)
+-> std::unique_ptr<TestWindowManagerPolicy>
+{
+    return std::make_unique<TestWindowManagerPolicy>(tools, *this);
+}
+
 void miral::TestServer::SetUp()
 {
 #if MIR_SERVER_VERSION < MIR_VERSION_NUMBER(0, 25, 0)
@@ -84,11 +99,32 @@ void miral::TestServer::SetUp()
                             return std::make_shared<mtf::HeadlessDisplayBufferCompositorFactory>();
                         });
 #endif
+
+                    server.override_the_window_manager_builder([this, &server](msh::FocusController* focus_controller)
+                        -> std::shared_ptr<msh::WindowManager>
+                        {
+                            auto const display_layout = server.the_shell_display_layout();
+
+#if MIR_SERVER_VERSION >= MIR_VERSION_NUMBER(0, 24, 0)
+                            auto const persistent_surface_store = server.the_persistent_surface_store();
+#else
+                            std::shared_ptr<mir::shell::PersistentSurfaceStore> const persistent_surface_store;
+#endif
+
+                            auto builder = [this](WindowManagerTools const& tools) -> std::unique_ptr<miral::WindowManagementPolicy>
+                                {
+                                    return build_window_manager_policy(tools);
+                                };
+
+                            auto wm = std::make_shared<miral::BasicWindowManager>(focus_controller, display_layout, persistent_surface_store, builder);
+                            window_manager = wm;
+                            return wm;
+                        });
                 };
 
             try
             {
-                runner.run_with({init, set_window_managment_policy<TestWindowManagerPolicy>(*this)});
+                runner.run_with({init});
             }
             catch (std::exception const& e)
             {
@@ -123,7 +159,7 @@ void miral::TestServer::TearDown()
     server_thread.stop();
 }
 
-auto miral::TestServer::connect_client(std::string name) -> toolkit::Connection
+auto miral::TestServer::connect_client(std::string name) -> mir::client::Connection
 {
     std::lock_guard<std::mutex> lock(mutex);
 
@@ -133,7 +169,21 @@ auto miral::TestServer::connect_client(std::string name) -> toolkit::Connection
     char connect_string[64] = {0};
     sprintf(connect_string, "fd://%d", dup(server_running->open_client_socket()));
 
-    return toolkit::Connection{mir_connect_sync(connect_string, name.c_str())};
+    return mir::client::Connection{mir_connect_sync(connect_string, name.c_str())};
+}
+
+void miral::TestServer::invoke_tools(std::function<void(WindowManagerTools& tools)> const& f)
+{
+    tools.invoke_under_lock([&]{f(tools); });
+}
+
+void miral::TestServer::invoke_window_manager(std::function<void(mir::shell::WindowManager& wm)> const& f)
+{
+    if (auto const wm = window_manager.lock())
+        f(*wm);
+    else
+        BOOST_THROW_EXCEPTION(std::runtime_error{"Server not running"});
+
 }
 
 void miral::TestRuntimeEnvironment::add_to_environment(char const* key, char const* value)
@@ -141,19 +191,6 @@ void miral::TestRuntimeEnvironment::add_to_environment(char const* key, char con
     env.emplace_back(key, value);
 }
 
-struct miral::TestServer::TestWindowManagerPolicy : CanonicalWindowManagerPolicy
-{
-    TestWindowManagerPolicy(WindowManagerTools const& tools, TestServer& test_fixture) :
-        CanonicalWindowManagerPolicy{tools}
-    {
-        test_fixture.tools = tools;
-    }
-
-    bool handle_keyboard_event(MirKeyboardEvent const*) override { return false; }
-    bool handle_pointer_event(MirPointerEvent const*) override { return false; }
-    bool handle_touch_event(MirTouchEvent const*) override { return false; }
-};
-
 using miral::TestServer;
 
 // Minimal test to ensure the server runs and exits
