@@ -1,5 +1,5 @@
 /*
- * Copyright © 2016 Canonical Ltd.
+ * Copyright © 2016-2017 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3,
@@ -20,24 +20,30 @@
 #define MIRAL_SHELL_TITLEBAR_WINDOW_MANAGER_H
 
 #include <miral/canonical_window_manager.h>
+#include <miral/workspace_policy.h>
 
 #include "spinner/splash.h"
 
 #include <chrono>
+#include <map>
 
 namespace miral { class InternalClientLauncher; }
 
 using namespace mir::geometry;
 
-class TitlebarProvider;
+class DecorationProvider;
 
-class TitlebarWindowManagerPolicy : public miral::CanonicalWindowManagerPolicy
+class TitlebarWindowManagerPolicy : public miral::CanonicalWindowManagerPolicy, miral::WorkspacePolicy
 {
 public:
-    TitlebarWindowManagerPolicy(miral::WindowManagerTools const& tools, SpinnerSplash const& spinner, miral::InternalClientLauncher const& launcher);
+    TitlebarWindowManagerPolicy(
+        miral::WindowManagerTools const& tools,
+        SpinnerSplash const& spinner,
+        miral::InternalClientLauncher const& launcher,
+        std::function<void()>& shutdown_hook);
     ~TitlebarWindowManagerPolicy();
 
-    virtual miral::WindowSpecification place_new_surface(
+    virtual miral::WindowSpecification place_new_window(
         miral::ApplicationInfo const& app_info, miral::WindowSpecification const& request_parameters) override;
 
     /** @name example event handling:
@@ -48,6 +54,8 @@ public:
      *  o Maximize/restore current window (to display size): Alt-F11
      *  o Maximize/restore current window (to display height): Shift-F11
      *  o Maximize/restore current window (to display width): Ctrl-F11
+     *  o Switch workspace . . . . . . . . . . : Meta-Alt-[F1|F2|F3|F4]
+     *  o Switch workspace taking active window: Meta-Ctrl-[F1|F2|F3|F4]
      *  @{ */
     bool handle_pointer_event(MirPointerEvent const* event) override;
     bool handle_touch_event(MirTouchEvent const* event) override;
@@ -60,9 +68,11 @@ public:
     void handle_window_ready(miral::WindowInfo& window_info) override;
     void advise_focus_lost(miral::WindowInfo const& info) override;
     void advise_focus_gained(miral::WindowInfo const& info) override;
-    void advise_state_change(miral::WindowInfo const& window_info, MirSurfaceState state) override;
+    void advise_state_change(miral::WindowInfo const& window_info, MirWindowState state) override;
     void advise_resize(miral::WindowInfo const& window_info, Size const& new_size) override;
     void advise_delete_window(miral::WindowInfo const& window_info) override;
+
+    void handle_modify_window(miral::WindowInfo& window_info, miral::WindowSpecification const& modifications) override;
     /** @} */
 
 protected:
@@ -74,7 +84,7 @@ protected:
         mir_input_event_modifier_meta;
 
 private:
-    void toggle(MirSurfaceState state);
+    void toggle(MirWindowState state);
 
     bool resize(miral::Window const& window, Point cursor, Point old_cursor);
 
@@ -92,7 +102,7 @@ private:
 
     SpinnerSplash const spinner;
 
-    std::unique_ptr<TitlebarProvider> const titlebar_provider;
+    std::unique_ptr<DecorationProvider> const decoration_provider;
 
     void end_resize();
 
@@ -104,6 +114,23 @@ private:
 
     // Workaround for lp:1627697
     std::chrono::steady_clock::time_point last_resize;
+
+    void advise_adding_to_workspace(
+        std::shared_ptr<miral::Workspace> const& workspace,
+        std::vector<miral::Window> const& windows) override;
+
+    // Switch workspace, taking window (if not null)
+    void switch_workspace_to(
+        std::shared_ptr<miral::Workspace> const& workspace,
+        miral::Window const& window = miral::Window{});
+
+    std::shared_ptr<miral::Workspace> active_workspace;
+    std::map<int, std::shared_ptr<miral::Workspace>> key_to_workspace;
+    std::map<std::shared_ptr<miral::Workspace>, miral::Window> workspace_to_active;
+
+    void apply_workspace_visible_to(miral::Window const& window);
+
+    void apply_workspace_hidden_to(miral::Window const& window);
 };
 
 #endif //MIRAL_SHELL_TITLEBAR_WINDOW_MANAGER_H
