@@ -25,13 +25,25 @@
 #include <mir/scene/surface.h>
 #include <mir/event_printer.h>
 
+#include <iomanip>
 #include <sstream>
 
 #define MIR_LOG_COMPONENT "miral::Window Management"
 #include <mir/log.h>
+#include <mir/report_exception.h>
 
 using mir::operator<<;
 
+#define MIRAL_TRACE_EXCEPTION \
+catch (std::exception const& x)\
+{\
+    std::stringstream out;\
+    mir::report_exception(out);\
+    mir::log_warning("%s throws %s", __func__, out.str().c_str());\
+    throw;\
+}
+
+
 namespace
 {
 std::string const null_ptr{"(null)"};
@@ -62,6 +74,22 @@ struct BracedItemStream
         first_field = false;
         return *this;
     }
+
+    auto append(char const* name, MirOrientationMode item) const -> BracedItemStream const&
+    {
+        auto const flags = out.flags();
+        auto const prec  = out.precision();
+        auto const fill  = out.fill();
+
+        if (!first_field) out << ", ";
+        out << name << '=' << std::showbase << std::internal << std::setfill('0') << std::setw(2) << std::hex << item;
+        first_field = false;
+
+        out.flags(flags);
+        out.precision(prec);
+        out.fill(fill);
+        return *this;
+    }
 };
 
 auto operator<< (std::ostream& out, miral::WindowSpecification::AspectRatio const& ratio) -> std::ostream&
@@ -88,8 +116,15 @@ auto dump_of(miral::Application const& application) -> std::string
 
 auto dump_of(std::vector<miral::Window> const& windows) -> std::string;
 
+inline auto operator!=(miral::WindowInfo::AspectRatio const& lhs, miral::WindowInfo::AspectRatio const& rhs)
+{
+    return lhs.width != rhs.width || lhs.height != rhs.height;
+}
+
 auto dump_of(miral::WindowInfo const& info) -> std::string
 {
+    using namespace mir::geometry;
+
     std::stringstream out;
     {
         BracedItemStream bout{out};
@@ -98,18 +133,18 @@ auto dump_of(miral::WindowInfo const& info) -> std::string
         APPEND(name);
         APPEND(type);
         APPEND(state);
-        APPEND(restore_rect);
+        if (info.state() != mir_window_state_restored) APPEND(restore_rect);
         if (std::shared_ptr<mir::scene::Surface> parent = info.parent())
             bout.append("parent", parent->name());
         bout.append("children", dump_of(info.children()));
-        APPEND(min_width);
-        APPEND(min_height);
-        APPEND(max_width);
-        APPEND(max_height);
-        APPEND(width_inc);
-        APPEND(height_inc);
-        APPEND(min_aspect);
-        APPEND(max_aspect);
+        if (info.min_width()  != Width{0}) APPEND(min_width);
+        if (info.min_height() != Height{0}) APPEND(min_height);
+        if (info.max_width()  != Width{std::numeric_limits<int>::max()}) APPEND(max_width);
+        if (info.max_height() != Height{std::numeric_limits<int>::max()}) APPEND(max_height);
+        if (info.width_inc()  != DeltaX{1}) APPEND(width_inc);
+        if (info.height_inc() != DeltaY{1}) APPEND(height_inc);
+        if (info.min_aspect() != miral::WindowInfo::AspectRatio{0U, std::numeric_limits<unsigned>::max()}) APPEND(min_aspect);
+        if (info.max_aspect() != miral::WindowInfo::AspectRatio{std::numeric_limits<unsigned>::max(), 0U}) APPEND(max_aspect);
         APPEND(preferred_orientation);
         APPEND(confine_pointer);
 
@@ -273,7 +308,7 @@ auto dump_of(MirPointerEvent const* event) -> std::string
     return out.str();
 }
 
-auto dump_of(MirSurfaceState state) -> std::string
+auto dump_of(MirWindowState state) -> std::string
 {
     std::stringstream out;
     out << state;
@@ -304,95 +339,105 @@ miral::WindowManagementTrace::WindowManagementTrace(
 }
 
 auto miral::WindowManagementTrace::count_applications() const -> unsigned int
-{
+try {
     log_input();
     auto const result = wrapped.count_applications();
     mir::log_info("%s -> %d", __func__, result);
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::for_each_application(std::function<void(miral::ApplicationInfo&)> const& functor)
-{
+try {
     log_input();
     mir::log_info("%s", __func__);
     trace_count++;
     wrapped.for_each_application(functor);
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::find_application(std::function<bool(ApplicationInfo const& info)> const& predicate)
 -> Application
-{
+try {
     log_input();
     auto result = wrapped.find_application(predicate);
     mir::log_info("%s -> %s", __func__, dump_of(result).c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::info_for(std::weak_ptr<mir::scene::Session> const& session) const -> ApplicationInfo&
-{
+try {
     log_input();
     auto& result = wrapped.info_for(session);
     mir::log_info("%s -> %s", __func__, result.application()->name().c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::info_for(std::weak_ptr<mir::scene::Surface> const& surface) const -> WindowInfo&
-{
+try {
     log_input();
     auto& result = wrapped.info_for(surface);
     mir::log_info("%s -> %s", __func__, result.name().c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::info_for(Window const& window) const -> WindowInfo&
-{
+try {
     log_input();
     auto& result = wrapped.info_for(window);
     mir::log_info("%s -> %s", __func__, result.name().c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::ask_client_to_close(miral::Window const& window)
-{
+try {
     log_input();
     mir::log_info("%s -> %s", __func__, dump_of(window).c_str());
     trace_count++;
     wrapped.ask_client_to_close(window);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::force_close(miral::Window const& window)
-{
+try {
     log_input();
     mir::log_info("%s -> %s", __func__, dump_of(window).c_str());
     trace_count++;
     wrapped.force_close(window);
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::active_window() const -> Window
-{
+try {
     log_input();
     auto result = wrapped.active_window();
     mir::log_info("%s -> %s", __func__, dump_of(result).c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::select_active_window(Window const& hint) -> Window
-{
+try {
     log_input();
     auto result = wrapped.select_active_window(hint);
     mir::log_info("%s hint=%s -> %s", __func__, dump_of(hint).c_str(), dump_of(result).c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::window_at(mir::geometry::Point cursor) const -> Window
-{
+try {
     log_input();
     auto result = wrapped.window_at(cursor);
     std::stringstream out;
@@ -401,9 +446,10 @@ auto miral::WindowManagementTrace::window_at(mir::geometry::Point cursor) const
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::active_display() -> mir::geometry::Rectangle const
-{
+try {
     log_input();
     auto result = wrapped.active_display();
     std::stringstream out;
@@ -412,35 +458,39 @@ auto miral::WindowManagementTrace::active_display() -> mir::geometry::Rectangle
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::info_for_window_id(std::string const& id) const -> WindowInfo&
-{
+try {
     log_input();
     auto& result = wrapped.info_for_window_id(id);
     mir::log_info("%s id=%s -> %s", __func__, id.c_str(), dump_of(result).c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::id_for_window(Window const& window) const -> std::string
-{
+try {
     log_input();
     auto result = wrapped.id_for_window(window);
     mir::log_info("%s window=%s -> %s", __func__, dump_of(window).c_str(), result.c_str());
     trace_count++;
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::place_and_size_for_state(
     WindowSpecification& modifications, WindowInfo const& window_info) const
-{
+try {
     log_input();
     mir::log_info("%s modifications=%s window_info=%s", __func__, dump_of(modifications).c_str(), dump_of(window_info).c_str());
     wrapped.place_and_size_for_state(modifications, window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::drag_active_window(mir::geometry::Displacement movement)
-{
+try {
     log_input();
     std::stringstream out;
     out << movement;
@@ -448,79 +498,156 @@ void miral::WindowManagementTrace::drag_active_window(mir::geometry::Displacemen
     trace_count++;
     wrapped.drag_active_window(movement);
 }
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::drag_window(Window const& window, mir::geometry::Displacement& movement)
+try {
+    log_input();
+    std::stringstream out;
+    out << movement;
+    mir::log_info("%s window=%s -> %s", __func__, dump_of(window).c_str(), out.str().c_str());
+    trace_count++;
+    wrapped.drag_window(window, movement);
+}
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::focus_next_application()
-{
+try {
     log_input();
     mir::log_info("%s", __func__);
     trace_count++;
     wrapped.focus_next_application();
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::focus_next_within_application()
-{
+try {
     log_input();
     mir::log_info("%s", __func__);
     trace_count++;
     wrapped.focus_next_within_application();
 }
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::focus_prev_within_application()
+try {
+    log_input();
+    mir::log_info("%s", __func__);
+    trace_count++;
+    wrapped.focus_prev_within_application();
+}
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::raise_tree(miral::Window const& root)
-{
+try {
     log_input();
     mir::log_info("%s root=%s", __func__, dump_of(root).c_str());
     trace_count++;
     wrapped.raise_tree(root);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::modify_window(
     miral::WindowInfo& window_info, miral::WindowSpecification const& modifications)
-{
+try {
     log_input();
     mir::log_info("%s window_info=%s, modifications=%s",
                   __func__, dump_of(window_info).c_str(), dump_of(modifications).c_str());
     trace_count++;
     wrapped.modify_window(window_info, modifications);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::invoke_under_lock(std::function<void()> const& callback)
-{
+try {
     mir::log_info("%s", __func__);
     wrapped.invoke_under_lock(callback);
 }
+MIRAL_TRACE_EXCEPTION
 
-auto miral::WindowManagementTrace::place_new_surface(
+auto miral::WindowManagementTrace::create_workspace() -> std::shared_ptr<Workspace>
+try {
+    mir::log_info("%s", __func__);
+    return wrapped.create_workspace();
+}
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::add_tree_to_workspace(
+    miral::Window const& window, std::shared_ptr<miral::Workspace> const& workspace)
+try {
+    mir::log_info("%s window=%s, workspace =%p", __func__, dump_of(window).c_str(), workspace.get());
+    wrapped.add_tree_to_workspace(window, workspace);
+}
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::remove_tree_from_workspace(
+    miral::Window const& window, std::shared_ptr<miral::Workspace> const& workspace)
+try {
+    mir::log_info("%s window=%s, workspace =%p", __func__, dump_of(window).c_str(), workspace.get());
+    wrapped.remove_tree_from_workspace(window, workspace);
+}
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::move_workspace_content_to_workspace(
+    std::shared_ptr<Workspace> const& to_workspace, std::shared_ptr<Workspace> const& from_workspace)
+try {
+    mir::log_info("%s to_workspace=%p, from_workspace=%p", __func__, to_workspace.get(), from_workspace.get());
+    wrapped.move_workspace_content_to_workspace(to_workspace, from_workspace);
+}
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::for_each_workspace_containing(
+    miral::Window const& window, std::function<void(std::shared_ptr<miral::Workspace> const&)> const& callback)
+try {
+    mir::log_info("%s window=%s", __func__, dump_of(window).c_str());
+    wrapped.for_each_workspace_containing(window, callback);
+}
+MIRAL_TRACE_EXCEPTION
+
+void miral::WindowManagementTrace::for_each_window_in_workspace(
+    std::shared_ptr<miral::Workspace> const& workspace, std::function<void(miral::Window const&)> const& callback)
+try {
+    mir::log_info("%s workspace =%p", __func__, workspace.get());
+    wrapped.for_each_window_in_workspace(workspace, callback);
+}
+MIRAL_TRACE_EXCEPTION
+
+auto miral::WindowManagementTrace::place_new_window(
     ApplicationInfo const& app_info,
     WindowSpecification const& requested_specification) -> WindowSpecification
-{
-    auto const result = policy->place_new_surface(app_info, requested_specification);
+try {
+    auto const result = policy->place_new_window(app_info, requested_specification);
     mir::log_info("%s app_info=%s, requested_specification=%s -> %s",
               __func__, dump_of(app_info).c_str(), dump_of(requested_specification).c_str(), dump_of(result).c_str());
     return result;
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::handle_window_ready(miral::WindowInfo& window_info)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->handle_window_ready(window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::handle_modify_window(
     miral::WindowInfo& window_info, miral::WindowSpecification const& modifications)
-{
+try {
     mir::log_info("%s window_info=%s, modifications=%s",
                   __func__, dump_of(window_info).c_str(), dump_of(modifications).c_str());
     policy->handle_modify_window(window_info, modifications);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::handle_raise_window(miral::WindowInfo& window_info)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->handle_raise_window(window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
 bool miral::WindowManagementTrace::handle_keyboard_event(MirKeyboardEvent const* event)
-{
+try {
     log_input = [event, this]
         {
             mir::log_info("handle_keyboard_event event=%s", dump_of(event).c_str());
@@ -529,9 +656,10 @@ bool miral::WindowManagementTrace::handle_keyboard_event(MirKeyboardEvent const*
 
     return policy->handle_keyboard_event(event);
 }
+MIRAL_TRACE_EXCEPTION
 
 bool miral::WindowManagementTrace::handle_touch_event(MirTouchEvent const* event)
-{
+try {
     log_input = [event, this]
         {
             mir::log_info("handle_touch_event event=%s", dump_of(event).c_str());
@@ -540,9 +668,10 @@ bool miral::WindowManagementTrace::handle_touch_event(MirTouchEvent const* event
 
     return policy->handle_touch_event(event);
 }
+MIRAL_TRACE_EXCEPTION
 
 bool miral::WindowManagementTrace::handle_pointer_event(MirPointerEvent const* event)
-{
+try {
     log_input = [event, this]
         {
             mir::log_info("handle_pointer_event event=%s", dump_of(event).c_str());
@@ -551,87 +680,101 @@ bool miral::WindowManagementTrace::handle_pointer_event(MirPointerEvent const* e
 
     return policy->handle_pointer_event(event);
 }
+MIRAL_TRACE_EXCEPTION
 
 auto miral::WindowManagementTrace::confirm_inherited_move(WindowInfo const& window_info, Displacement movement)
 -> Rectangle
-{
+try {
     std::stringstream out;
     out << movement;
     mir::log_info("%s window_info=%s, movement=%s", __func__, dump_of(window_info).c_str(), out.str().c_str());
 
     return policy->confirm_inherited_move(window_info, movement);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_begin()
-{
+try {
     log_input = []{};
     trace_count.store(0);
     policy->advise_begin();
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_end()
-{
+try {
     if (trace_count.load() > 0)
         mir::log_info("====");
     policy->advise_end();
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_new_app(miral::ApplicationInfo& application)
-{
+try {
     mir::log_info("%s application=%s", __func__, dump_of(application).c_str());
     policy->advise_new_app(application);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_delete_app(miral::ApplicationInfo const& application)
-{
+try {
     mir::log_info("%s application=%s", __func__, dump_of(application).c_str());
     policy->advise_delete_app(application);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_new_window(miral::WindowInfo const& window_info)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->advise_new_window(window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_focus_lost(miral::WindowInfo const& window_info)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->advise_focus_lost(window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_focus_gained(miral::WindowInfo const& window_info)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->advise_focus_gained(window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
-void miral::WindowManagementTrace::advise_state_change(miral::WindowInfo const& window_info, MirSurfaceState state)
-{
+void miral::WindowManagementTrace::advise_state_change(miral::WindowInfo const& window_info, MirWindowState state)
+try {
     mir::log_info("%s window_info=%s, state=%s", __func__, dump_of(window_info).c_str(), dump_of(state).c_str());
     policy->advise_state_change(window_info, state);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_move_to(miral::WindowInfo const& window_info, mir::geometry::Point top_left)
-{
+try {
     mir::log_info("%s window_info=%s, top_left=%s", __func__, dump_of(window_info).c_str(), dump_of(top_left).c_str());
     policy->advise_move_to(window_info, top_left);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_resize(miral::WindowInfo const& window_info, mir::geometry::Size const& new_size)
-{
+try {
     mir::log_info("%s window_info=%s, new_size=%s", __func__, dump_of(window_info).c_str(), dump_of(new_size).c_str());
     policy->advise_resize(window_info, new_size);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_delete_window(miral::WindowInfo const& window_info)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(window_info).c_str());
     policy->advise_delete_window(window_info);
 }
+MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_raise(std::vector<miral::Window> const& windows)
-{
+try {
     mir::log_info("%s window_info=%s", __func__, dump_of(windows).c_str());
     policy->advise_raise(windows);
 }
+MIRAL_TRACE_EXCEPTION
